package com.muyu.parsing.mqtt;

import com.muyu.common.kafka.constants.KafkaConstants;
import com.muyu.parsing.domain.SysCarMessage;
import com.muyu.parsing.service.impl.SysCarMessageServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.producer.KafkaProducer;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.eclipse.paho.client.mqttv3.*;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CompletableFuture;


/**
 * mqtt
 *
 * @ClassName MqttTest
 * @Description 描述
 * @Author Chen
 * @Date 2024/9/28 23:49
 */
@Slf4j
@Component
public class MqttTest {
    private static final Integer ID = 1;
    private static final Integer CODE = 1;

    @Resource
    private KafkaProducer<String, String> kafkaProducer;
    @Resource
    private SysCarMessageServiceImpl sysCarMessageService;

    @PostConstruct
    public void Test() {
        String topic = "vehicle";
        String content = "Message from MqttPublishSample";
        int qos = 2;
        String broker = "tcp://106.15.136.7:1883";
        String clientId = "JavaSample";

        try {
            // 第三个参数为空，默认持久化策略
            MqttClient sampleClient = new MqttClient(broker, clientId);
            MqttConnectOptions connOpts = new MqttConnectOptions();
            connOpts.setCleanSession(true);
            System.out.println("Connecting to broker: " + broker);
            sampleClient.connect(connOpts);
            sampleClient.subscribe(topic, 0);
            sampleClient.setCallback(new MqttCallback() {
                // 连接丢失
                @Override
                public void connectionLost(Throwable throwable) {

                }

                // 连接成功
                @Override
                public void messageArrived(String s, MqttMessage mqttMessage) throws Exception {
                    List<SysCarMessage> list = sysCarMessageService.selectSysCarMessageLists(ID, CODE);
                    String string = new String(mqttMessage.getPayload());
                    log.info(new String(mqttMessage.getPayload()));
                    String[] test = string.split(" ");
                    String[] results = new String[list.size()];
                    List<CompletableFuture<String>> futures = new ArrayList<>();
                    for (SysCarMessage carMessage : list) {
                        futures.add(CompletableFuture.supplyAsync(() -> {
                            int start = Integer.parseInt(carMessage.getMessageStartIndex()) - 1;
                            int end = Integer.parseInt(carMessage.getMessageEndIndex());
                            StringBuilder hexBuilder = new StringBuilder();
                            for (int i = start; i < end; i++) {
                                hexBuilder.append(test[i]);
                            }
                            String hex = hexBuilder.toString();
                            char[] result = new char[hex.length() / 2];
                            for (int x = 0; x < hex.length(); x += 2) {
                                int high = Character.digit(hex.charAt(x), 16);
                                int low = Character.digit(hex.charAt(x + 1), 16);
                                result[x / 2] = (char) ((high << 4) + low);
                            }
                            return new String(result);
                        }));
                    }
                    for (int i = 0; i < futures.size(); i++) {
                        results[i] = futures.get(i).get();
                    }
                    log.info("======================={}", results);
                    String jsonString = """
                            [{
                                "key": "vin",
                                "label": "VIN码",
                                "type": "String",
                                "value": "vin131413534474"
                            },{
                                "key": "timestamp",
                                "label": "时间戳",
                                "type": "String",
                                "value": "1727525252127"
                            },{
                                "key": "latitude",
                                "label": "纬度",
                                "type": "String",
                                "value": "66.898"
                            },{
                                "key": "longitude",
                                "label": "经度",
                                "type": "String",
                                "value": "99.124"
                            }]""";

                    ProducerRecord<String, String> producerRecord = new ProducerRecord<>(KafkaConstants.KafkaTopic, jsonString);
                    kafkaProducer.send(producerRecord);
                    log.info("消息发送成功：{}", jsonString);

                }

                // 接收信息
                @Override
                public void deliveryComplete(IMqttDeliveryToken iMqttDeliveryToken) {

                }

            });
        } catch (MqttException me) {
            System.out.println("reason " + me.getReasonCode());
            System.out.println("msg " + me.getMessage());
            System.out.println("loc " + me.getLocalizedMessage());
            System.out.println("cause " + me.getCause());
            System.out.println("excep " + me);
            me.printStackTrace();
        }
    }
}
